@@ -1,5 +1,5 @@
 // Symphonia
-// Copyright (c) 2020 The Project Symphonia Developers.
+// Copyright (c) 2019-2021 The Project Symphonia Developers.
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,15 +10,15 @@ use std::num::Wrapping;
 
 use symphonia_core::audio::{AudioBuffer, AudioBufferRef, AsAudioBufferRef};
 use symphonia_core::audio::{Signal, SignalSpec};
-use symphonia_core::codecs::{CODEC_TYPE_FLAC, CodecParameters, CodecDescriptor};
-use symphonia_core::codecs::{Decoder, DecoderOptions};
+use symphonia_core::codecs::{CODEC_TYPE_FLAC, CodecParameters, CodecDescriptor, VerificationCheck};
+use symphonia_core::codecs::{Decoder, DecoderOptions, FinalizeResult};
 use symphonia_core::errors::{Result, decode_error, unsupported_error};
 use symphonia_core::formats::Packet;
-use symphonia_core::io::{BitStream, BitStreamLtr};
+use symphonia_core::io::{ReadBitsLtr, BitReaderLtr};
 use symphonia_core::support_codec;
 use symphonia_core::util::bits::sign_extend_leq32_to_i32;
 
-use log::info;
+use log::{debug, log_enabled, warn};
 
 use super::frame::*;
 use super::validate::Validator;
@@ -93,25 +93,25 @@ impl Decoder for FlacDecoder {
         // Consider supporting this.
         let frames = match params.max_frames_per_packet {
             Some(frames) => frames,
-            None => return unsupported_error("variable frames per packet are unsupported"),
+            None => return unsupported_error("flac: variable frames per packet are unsupported"),
         };
 
         let spec = {
             let sample_rate = match params.sample_rate {
                 Some(rate) => rate,
-                None       => return unsupported_error("variable sample rate is unsupported"),
+                None => return unsupported_error("flac: variable sample rate is unsupported"),
             };
 
             let channels = match params.channels {
                 Some(channels) => channels,
-                None           => return unsupported_error("dynamic channels are unsupported"),
+                None => return unsupported_error("flac: dynamic channels are unsupported"),
             };
 
             SignalSpec::new(sample_rate, channels)
         };
 
         if !params.packet_data_integrity {
-            return unsupported_error("packet integrity is required");
+            return unsupported_error("flac: packet integrity is required");
         }
 
         Ok(FlacDecoder {
@@ -126,12 +126,16 @@ impl Decoder for FlacDecoder {
         &[ support_codec!(CODEC_TYPE_FLAC, "flac", "Free Lossless Audio Codec") ]
     }
 
+    fn reset(&mut self) {
+        // No state is stored between packets, therefore do nothing.
+    }
+
     fn codec_params(&self) -> &CodecParameters {
         &self.params
     }
 
     fn decode(&mut self, packet: &Packet) -> Result<AudioBufferRef<'_>> {
-        let mut reader = packet.as_buf_stream();
+        let mut reader = packet.as_buf_reader();
 
         // Synchronize to a frame and get the synchronization code.
         let sync = sync_frame(&mut reader)?;
@@ -143,7 +147,7 @@ impl Decoder for FlacDecoder {
         let bits_per_sample = if let Some(bps) = header.bits_per_sample { bps }
                               else if let Some(bps) = self.params.bits_per_sample { bps }
                               else {
-                                  return decode_error("bits per sample not provided");
+                                  return decode_error("flac: bits per sample not provided");
                               };
 
         // trace!("frame: [{:?}] strategy={:?}, n_samples={}, bps={}, channels={:?}",
@@ -160,7 +164,7 @@ impl Decoder for FlacDecoder {
         // Only Bitstream reading for subframes.
         {
             // Sub-frames don't have any byte-aligned content, so use a BitReader.
-            let mut bs = BitStreamLtr::new(&mut reader);
+            let mut bs = BitReaderLtr::new(reader.read_buf_bytes_available_ref());
 
             // Read each subframe based on the channel assignment into a planar buffer.
             match header.channel_assignment {
@@ -172,28 +176,28 @@ impl Decoder for FlacDecoder {
                 // For Left/Side, Mid/Side, and Right/Side channel configurations, the Side
                 // (Difference) channel requires an extra bit per sample.
                 ChannelAssignment::LeftSide => {
-                    let (mut left, mut side) = self.buf.chan_pair_mut(0, 1);
+                    let (left, side) = self.buf.chan_pair_mut(0, 1);
 
-                    read_subframe(&mut bs, bits_per_sample, &mut left)?;
-                    read_subframe(&mut bs, bits_per_sample + 1, &mut side)?;
+                    read_subframe(&mut bs, bits_per_sample, left)?;
+                    read_subframe(&mut bs, bits_per_sample + 1, side)?;
 
-                    decorrelate_left_side(&left, &mut side);
+                    decorrelate_left_side(left, side);
                 },
                 ChannelAssignment::MidSide => {
-                    let (mut mid, mut side) = self.buf.chan_pair_mut(0, 1);
+                    let (mid, side) = self.buf.chan_pair_mut(0, 1);
 
-                    read_subframe(&mut bs, bits_per_sample, &mut mid)?;
-                    read_subframe(&mut bs, bits_per_sample + 1, &mut side)?;
+                    read_subframe(&mut bs, bits_per_sample, mid)?;
+                    read_subframe(&mut bs, bits_per_sample + 1, side)?;
 
-                    decorrelate_mid_side(&mut mid, &mut side);
+                    decorrelate_mid_side(mid, side);
                 },
                 ChannelAssignment::RightSide => {
-                    let (mut side, mut right) = self.buf.chan_pair_mut(0, 1);
+                    let (side, right) = self.buf.chan_pair_mut(0, 1);
 
-                    read_subframe(&mut bs, bits_per_sample + 1, &mut side)?;
-                    read_subframe(&mut bs, bits_per_sample, &mut right)?;
+                    read_subframe(&mut bs, bits_per_sample + 1, side)?;
+                    read_subframe(&mut bs, bits_per_sample, right)?;
 
-                    decorrelate_right_side(&right, &mut side);
+                    decorrelate_right_side(right, side);
                 }
             }
         }
@@ -214,10 +218,38 @@ impl Decoder for FlacDecoder {
         Ok(self.buf.as_audio_buffer_ref())
     }
 
-    fn close(&mut self) {
+    fn finalize(&mut self) -> FinalizeResult {
+        let mut result: FinalizeResult = Default::default();
+
+        // If verifying...
         if self.is_validating {
-            info!("output md5 = {:x?}", self.validator.md5());
+            // Try to get the expected MD5 checksum and compare it against the decoded checksum.
+            if let Some(VerificationCheck::Md5(expected)) = self.params.verification_check {
+                let decoded = self.validator.md5();
+
+                // Only generate the expected and decoded MD5 checksum strings if logging is
+                // enabled at the debug level.
+                if log_enabled!(log::Level::Debug) {
+                    use std::fmt::Write;
+
+                    let mut expected_s = String::with_capacity(32);
+                    let mut decoded_s = String::with_capacity(32);
+
+                    expected.iter().for_each(|b| write!(expected_s, "{:02x}", b).unwrap());
+                    decoded.iter().for_each(|b| write!(decoded_s, "{:02x}", b).unwrap());
+
+                    debug!("verification: expected md5 = {}", expected_s);
+                    debug!("verification: decoded md5  = {}", decoded_s);
+                }
+
+                result.verify_ok = Some(decoded == expected)
+            }
+            else {
+                warn!("verification requested but the expected md5 checksum was not provided");
+            }
         }
+
+        result
     }
 
 }
@@ -232,11 +264,11 @@ enum SubFrameType {
     Linear(u32),
 }
 
-fn read_subframe<B: BitStream>(bs: &mut B, frame_bps: u32, buf: &mut [i32]) -> Result<()> {
+fn read_subframe<B: ReadBitsLtr>(bs: &mut B, frame_bps: u32, buf: &mut [i32]) -> Result<()> {
 
     // First sub-frame bit must always 0.
     if bs.read_bit()? {
-        return decode_error("subframe padding is not 0");
+        return decode_error("flac: subframe padding is not 0");
     }
 
     // Next 6 bits designate the sub-frame type.
@@ -249,13 +281,13 @@ fn read_subframe<B: BitStream>(bs: &mut B, frame_bps: u32, buf: &mut [i32]) -> R
             let order = subframe_type_enc & 0x07;
             // The Fixed Predictor only supports orders between 0 and 4.
             if order > 4 {
-                return decode_error("fixed predictor orders of greater than 4 are invalid");
+                return decode_error("flac: fixed predictor orders of greater than 4 are invalid");
             }
             SubFrameType::FixedLinear(order)
         },
         0x20..=0x3f => SubFrameType::Linear((subframe_type_enc & 0x1f) + 1),
         _ => {
-            return decode_error("subframe type set to reserved value");
+            return decode_error("flac: subframe type set to reserved value");
         }
     };
 
@@ -263,7 +295,7 @@ fn read_subframe<B: BitStream>(bs: &mut B, frame_bps: u32, buf: &mut [i32]) -> R
     // bits per sample in the audio sub-block. If the bit is set, unary decode the number of
     // dropped bits per sample.
     let dropped_bps = if bs.read_bit()? {
-        bs.read_unary()? + 1
+        bs.read_unary_zeros()? + 1
     }
     else {
         0
@@ -303,7 +335,7 @@ fn samples_shl(shift: u32, buf: &mut [i32]) {
     }
 }
 
-fn decode_constant<B: BitStream>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Result<()> {
+fn decode_constant<B: ReadBitsLtr>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Result<()> {
     let const_sample = sign_extend_leq32_to_i32(bs.read_bits_leq32(bps)?, bps);
 
     for sample in buf.iter_mut() {
@@ -313,7 +345,7 @@ fn decode_constant<B: BitStream>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Resul
     Ok(())
 }
 
-fn decode_verbatim<B: BitStream>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Result<()> {
+fn decode_verbatim<B: ReadBitsLtr>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Result<()> {
     for sample in buf.iter_mut() {
         *sample = sign_extend_leq32_to_i32(bs.read_bits_leq32(bps)?, bps);
     }
@@ -321,7 +353,7 @@ fn decode_verbatim<B: BitStream>(bs: &mut B, bps: u32, buf: &mut [i32]) -> Resul
     Ok(())
 }
 
-fn decode_fixed_linear<B: BitStream>(
+fn decode_fixed_linear<B: ReadBitsLtr>(
     bs: &mut B,
     bps: u32,
     order: u32,
@@ -343,7 +375,7 @@ fn decode_fixed_linear<B: BitStream>(
     Ok(())
 }
 
-fn decode_linear<B: BitStream>(bs: &mut B, bps: u32, order: u32, buf: &mut [i32]) -> Result<()> {
+fn decode_linear<B: ReadBitsLtr>(bs: &mut B, bps: u32, order: u32, buf: &mut [i32]) -> Result<()> {
     // The order of the Linear Predictor should be between 1 and 32.
     debug_assert!(order > 0 && order <= 32);
 
@@ -353,7 +385,7 @@ fn decode_linear<B: BitStream>(bs: &mut B, bps: u32, order: u32, buf: &mut [i32]
     // Quantized linear predictor (QLP) coefficients precision in bits.
     let qlp_precision = bs.read_bits_leq32(4)? + 1;
     if qlp_precision > 15 {
-        return decode_error("QLP precision set to reserved value");
+        return decode_error("flac: qlp precision set to reserved value");
     }
 
     // QLP coefficients bit shift [-16, 15].
@@ -411,13 +443,13 @@ fn decode_linear<B: BitStream>(bs: &mut B, bps: u32, order: u32, buf: &mut [i32]
         }
     }
     else {
-        return unsupported_error("LPC shifts less than 0 are not supported");
+        return unsupported_error("flac: lpc shifts less than 0 are not supported");
     }
 
     Ok(())
 }
 
-fn decode_residual<B: BitStream>(
+fn decode_residual<B: ReadBitsLtr>(
     bs: &mut B,
     n_prelude_samples: u32,
     buf: &mut [i32]
@@ -432,7 +464,7 @@ fn decode_residual<B: BitStream>(
         0x0 => 4,
         0x1 => 5,
         _ => {
-            return decode_error("residual method set to reserved value");
+            return decode_error("flac: residual method set to reserved value");
         }
     };
 
@@ -453,12 +485,12 @@ fn decode_residual<B: BitStream>(
     // minus the number of warm-up samples (which is the predictor order). Ensure the number of
     // samples in these types of partitions cannot be negative.
     if n_prelude_samples as usize > n_partition_samples {
-        return decode_error("residual partition too small for given predictor order");
+        return decode_error("flac: residual partition too small for given predictor order");
     }
 
     // Ensure that the sum of all partition lengths equal the block size.
     if n_partitions * n_partition_samples != buf.len() {
-        return decode_error("block size is not same as encoded residual");
+        return decode_error("flac: block size is not same as encoded residual");
     }
 
     // trace!("\t\tresidual: n_partitions={}, n_partition_samples={}, n_prelude_samples={}",
@@ -481,7 +513,7 @@ fn decode_residual<B: BitStream>(
     Ok(())
 }
 
-fn decode_rice_partition<B: BitStream>(
+fn decode_rice_partition<B: ReadBitsLtr>(
     bs: &mut B,
     param_bit_width: u32,
     buf: &mut [i32]
@@ -499,7 +531,7 @@ fn decode_rice_partition<B: BitStream>(
 
         // Read each rice encoded residual and store in buffer.
         for sample in buf.iter_mut() {
-            let q = bs.read_unary()?;
+            let q = bs.read_unary_zeros()?;
             let r = bs.read_bits_leq32(rice_param)?;
             *sample = rice_signed_to_i32((q << rice_param) | r);
         }
